@@ -26,4 +26,4 @@ BEGIN
 
   RETURN result;
 END;
-$_$ LANGUAGE plpgsql;
+$_$ LANGUAGE plpgsql IMMUTABLE COST 10;
