@@ -261,64 +261,43 @@ proj_cs_get_simplecs(const PJ *pj_crs)
 	return NULL;
 }
 
-
-#define STR_EQUALS(A, B) strcmp((A), (B)) == 0
-#define STR_IEQUALS(A, B) (strcasecmp((A), (B)) == 0)
-#define STR_ISTARTS(A, B) (strncasecmp((A), (B), strlen((B))) == 0)
-
 static uint8_t
 proj_crs_is_swapped(const PJ *pj_crs)
 {
-    int axis_count;
-    PJ *pj_cs = proj_cs_get_simplecs(pj_crs);
-    if (!pj_cs)
-        lwerror("%s: proj_cs_get_simplecs returned NULL", __func__);
+	PJ *pj_cs;
+	uint8_t rv = LW_FALSE;
 
-    axis_count = proj_cs_get_axis_count(NULL, pj_cs);
-    if (axis_count >= 2)
-    {
-        const char *out_name1, *out_abbrev1, *out_direction1;
-        const char *out_name2, *out_abbrev2, *out_direction2;
-        /* Read first axis */
-        proj_cs_get_axis_info(NULL,
-            pj_cs, 0,
-            &out_name1, &out_abbrev1, &out_direction1,
-            NULL, NULL, NULL, NULL);
-        /* Read second axis */
-        proj_cs_get_axis_info(NULL,
-            pj_cs, 1,
-            &out_name2, &out_abbrev2, &out_direction2,
-            NULL, NULL, NULL, NULL);
+	pj_cs = proj_cs_get_simplecs(pj_crs);
+	if (!pj_cs) {
+		lwerror("%s: proj_cs_get_simplecs returned NULL", __func__);
+	}
+	int axis_count = proj_cs_get_axis_count(NULL, pj_cs);
+	if (axis_count > 0)
+	{
+		const char *out_name, *out_abbrev, *out_direction;
+		double out_unit_conv_factor;
+		const char *out_unit_name, *out_unit_auth_name, *out_unit_code;
+		/* Read only first axis */
+		proj_cs_get_axis_info(NULL,
+				      pj_cs,
+				      0,
+				      &out_name,
+				      &out_abbrev,
+				      &out_direction,
+				      &out_unit_conv_factor,
+				      &out_unit_name,
+				      &out_unit_auth_name,
+				      &out_unit_code);
 
-        proj_destroy(pj_cs);
-
-        /* Directions agree, this is a northing/easting CRS, so reverse it */
-        if(out_direction1 && STR_IEQUALS(out_direction1, "north") &&
-           out_direction2 && STR_IEQUALS(out_direction2, "east") )
-        {
-            return LW_TRUE;
-        }
-
-        /* Oddball case? Both axes north / both axes south, swap */
-        if(out_direction1 && out_direction2 &&
-           ((STR_IEQUALS(out_direction1, "north") && STR_IEQUALS(out_direction2, "north")) ||
-            (STR_IEQUALS(out_direction1, "south") && STR_IEQUALS(out_direction2, "south"))) &&
-           out_name1 && STR_ISTARTS(out_name1, "northing")  &&
-           out_name2 && STR_ISTARTS(out_name2, "easting"))
-        {
-            return LW_TRUE;
-        }
-
-        /* Any lat/lon system with Lat in first axis gets swapped */
-        if (STR_ISTARTS(out_abbrev1, "Lat"))
-            return LW_TRUE;
-
-        return LW_FALSE;
-    }
-
-    /* Failed the axis count test, leave quietly */
-    proj_destroy(pj_cs);
-    return LW_FALSE;
+		/* Only swap Lat/Lon systems */
+		/* Use whatever ordering planar systems default to */
+		if (strcasecmp(out_abbrev, "Lat") == 0)
+			rv = LW_TRUE;
+		else
+			rv = LW_FALSE;
+	}
+	proj_destroy(pj_cs);
+	return rv;
 }
 
 LWPROJ *
@@ -405,7 +384,6 @@ lwgeom_transform_from_str(LWGEOM *geom, const char* instr, const char* outstr)
 		PJ *pj_in = proj_create(NULL, instr);
 		if (!pj_in)
 		{
-			proj_errno_reset(NULL);
 			lwerror("could not parse proj string '%s'", instr);
 		}
 		proj_destroy(pj_in);
@@ -413,7 +391,6 @@ lwgeom_transform_from_str(LWGEOM *geom, const char* instr, const char* outstr)
 		PJ *pj_out = proj_create(NULL, outstr);
 		if (!pj_out)
 		{
-			proj_errno_reset(NULL);
 			lwerror("could not parse proj string '%s'", outstr);
 		}
 		proj_destroy(pj_out);
@@ -519,18 +496,20 @@ ptarray_transform(POINTARRAY *pa, LWPROJ *pj)
 	{
 		/* For single points it's faster to call proj_trans */
 		PJ_XYZT v = {pa_double[0], pa_double[1], has_z ? pa_double[2] : 0.0, 0.0};
-		PJ_COORD t = proj_trans(pj->pj, PJ_FWD, (PJ_COORD)v);
+		PJ_COORD c;
+		c.xyzt = v;
+		PJ_COORD t = proj_trans(pj->pj, PJ_FWD, c);
 
-		int pj_errno_val = proj_errno(pj->pj);
+		int pj_errno_val = proj_errno_reset(pj->pj);
 		if (pj_errno_val)
 		{
 			lwerror("transform: %s (%d)", proj_errno_string(pj_errno_val), pj_errno_val);
 			return LW_FAILURE;
 		}
-		pa_double[0] = ((PJ_XYZT)t.xyzt).x;
-		pa_double[1] = ((PJ_XYZT)t.xyzt).y;
+		pa_double[0] = (t.xyzt).x;
+		pa_double[1] = (t.xyzt).y;
 		if (has_z)
-			pa_double[2] = ((PJ_XYZT)t.xyzt).z;
+			pa_double[2] = (t.xyzt).z;
 	}
 	else
 	{
@@ -564,7 +543,7 @@ ptarray_transform(POINTARRAY *pa, LWPROJ *pj)
 			return LW_FAILURE;
 		}
 
-		int pj_errno_val = proj_errno(pj->pj);
+		int pj_errno_val = proj_errno_reset(pj->pj);
 		if (pj_errno_val)
 		{
 			lwerror("transform: %s (%d)", proj_errno_string(pj_errno_val), pj_errno_val);
