@@ -46,16 +46,15 @@
 *
 **********************************************************************/
 
+#include "postgres.h"
+#include "executor/spi.h"
+#include "utils/builtins.h"
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
-#include "postgres.h"
-#include "executor/spi.h"
-#include "utils/builtins.h"
-
 #include "../postgis_config.h"
 #include "lwgeom_pg.h"
 #include "liblwgeom.h"
@@ -118,7 +117,7 @@ Datum geom_from_gml(PG_FUNCTION_ARGS)
 	 * That function requires access to spatial_ref_sys, so in order to have it ready we need to ensure
 	 * the internal cache is initialized
 	 */
-	postgis_initialize_cache(fcinfo);
+	postgis_initialize_cache();
 #endif
 	lwgeom = lwgeom_from_gml(xml, xml_size);
 	if ( root_srid != SRID_UNKNOWN )
@@ -131,28 +130,6 @@ Datum geom_from_gml(PG_FUNCTION_ARGS)
 }
 
 
-static inline bool
-is_gml_element(xmlNodePtr xn, const char *gml_name)
-{
-	char *colon_pos;
-	char *node_name;
-
-	/* Not an element node, can't do anything */
-	if (!xn || xn->type != XML_ELEMENT_NODE)
-		return false;
-
-	/* If there's a colon in the element name, */
-	/* move past it before checking for equality with */
-	/* the element name we are looking for */
-	node_name = (char*)xn->name;
-	colon_pos = strchr(node_name, ':');
-	if (colon_pos)
-		node_name = colon_pos + 1;
-
-	return strcmp(node_name, gml_name) == 0;
-}
-
-
 /**
  * Return false if current element namespace is not a GML one
  * Return true otherwise.
@@ -202,10 +179,9 @@ static bool is_gml_namespace(xmlNodePtr xnode, bool is_strict)
  * Retrieve a GML property from a node or NULL otherwise
  * Respect namespaces if presents in the node element
  */
-static xmlChar *gmlGetProp(xmlNodePtr xnode, const char *charProp)
+static xmlChar *gmlGetProp(xmlNodePtr xnode, xmlChar *prop)
 {
 	xmlChar *value;
-	xmlChar *prop = (xmlChar*)charProp;
 
 	if (!is_gml_namespace(xnode, true))
 		return xmlGetProp(xnode, prop);
@@ -308,7 +284,7 @@ static xmlNodePtr get_xlink_node(xmlNodePtr xnode)
 	for (node = xnode ; node != NULL ; node = node->parent)
 	{
 		if (node->type != XML_ELEMENT_NODE) continue;
-		node_id = gmlGetProp(node, "id");
+		node_id = gmlGetProp(node, (xmlChar *) "id");
 		if (node_id != NULL)
 		{
 			if (!xmlStrcmp(node_id, p))
@@ -407,25 +383,25 @@ gml_reproject_pa(POINTARRAY *pa, int32_t epsg_in, int32_t epsg_out)
 
 
 /**
- * Return 1 if given srid is planar (0 otherwise, i.e geocentric srid)
- * Return -1 if srid is not in spatial_ref_sys
+ * Return 1 if the SRS definition from the authority has a GIS friendly order,
+ * that is easting,northing. This is typically true for most projected CRS
+ * (but not all!), and this is false for EPSG geographic CRS.
  */
 static int
-gml_is_srid_planar(int32_t srid)
+gml_is_srs_axis_order_gis_friendly(int32_t srid)
 {
-	char *result;
+	char *srtext;
 	char query[256];
-	int is_planar, err;
+	int is_axis_order_gis_friendly, err;
 
 	if (SPI_OK_CONNECT != SPI_connect ())
-		lwpgerror("gml_is_srid_planar: could not connect to SPI manager");
+		lwpgerror("gml_is_srs_axis_order_gis_friendly: could not connect to SPI manager");
 
-	/* A way to find if this projection is planar or geocentric */
-	sprintf(query, "SELECT position('+units=m ' in proj4text) \
+	sprintf(query, "SELECT srtext \
                         FROM spatial_ref_sys WHERE srid='%d'", srid);
 
 	err = SPI_exec(query, 1);
-	if (err < 0) lwpgerror("gml_is_srid_planar: error executing query %d", err);
+	if (err < 0) lwpgerror("gml_is_srs_axis_order_gis_friendly: error executing query %d", err);
 
 	/* No entry in spatial_ref_sys */
 	if (SPI_processed <= 0)
@@ -434,11 +410,49 @@ gml_is_srid_planar(int32_t srid)
 		return -1;
 	}
 
-	result = SPI_getvalue(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1);
-	is_planar = atoi(result);
+	srtext = SPI_getvalue(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1);
+
+	is_axis_order_gis_friendly = 1;
+	if (srtext && srtext[0] != '\0')
+	{
+		char* ptr;
+		char* srtext_horizontal = (char*) malloc(strlen(srtext) + 1);
+		strcpy(srtext_horizontal, srtext);
+
+		/* Remove the VERT_CS part if we are in a COMPD_CS */
+		ptr = strstr(srtext_horizontal, ",VERT_CS[");
+		if (ptr)
+			*ptr = '\0';
+
+		if( strstr(srtext_horizontal, "AXIS[") == NULL &&
+		    strstr(srtext_horizontal, "GEOCCS[") == NULL )
+		{
+			/* If there is no axis definition, then due to how GDAL < 3
+			* generated the WKT, this means that the axis order is not
+			* GIS friendly */
+			is_axis_order_gis_friendly = 0;
+		}
+		else if( strstr(srtext_horizontal,
+		           "AXIS[\"Latitude\",NORTH],AXIS[\"Longitude\",EAST]") != NULL )
+		{
+			is_axis_order_gis_friendly = 0;
+		}
+		else if( strstr(srtext_horizontal,
+		           "AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]") != NULL )
+		{
+			is_axis_order_gis_friendly = 0;
+		}
+		else if( strstr(srtext_horizontal,
+		           "AXIS[\"geodetic latitude (Lat)\",north,ORDER[1]") != NULL )
+		{
+			is_axis_order_gis_friendly = 0;
+		}
+
+		free(srtext_horizontal);
+	}
 	SPI_finish();
 
-	return is_planar;
+	return is_axis_order_gis_friendly;
 }
 
 
@@ -448,14 +462,14 @@ gml_is_srid_planar(int32_t srid)
 static void parse_gml_srs(xmlNodePtr xnode, gmlSrs *srs)
 {
 	char *p;
-	int is_planar;
+	int is_axis_order_gis_friendly;
 	xmlNodePtr node;
 	xmlChar *srsname;
-	bool latlon = false;
+	bool honours_authority_axis_order = false;
 	char sep = ':';
 
 	node = xnode;
-	srsname = gmlGetProp(node, "srsName");
+	srsname = gmlGetProp(node, (xmlChar *) "srsName");
 	/*printf("srsname %s\n",srsname);*/
 	if (!srsname)
 	{
@@ -489,20 +503,20 @@ static void parse_gml_srs(xmlNodePtr xnode, gmlSrs *srs)
 		if (!strncmp((char *) srsname, "EPSG:", 5))
 		{
 			sep = ':';
-			latlon = false;
+			honours_authority_axis_order = false;
 		}
 		else if (!strncmp((char *) srsname, "urn:ogc:def:crs:EPSG:", 21)
 		         || !strncmp((char *) srsname, "urn:x-ogc:def:crs:EPSG:", 23)
 		         || !strncmp((char *) srsname, "urn:EPSG:geographicCRS:", 23))
 		{
 			sep = ':';
-			latlon = true;
+			honours_authority_axis_order = true;
 		}
 		else if (!strncmp((char *) srsname,
 		                  "http://www.opengis.net/gml/srs/epsg.xml#", 40))
 		{
 			sep = '#';
-			latlon = false;
+			honours_authority_axis_order = false;
 		}
 		else gml_lwpgerror("unknown spatial reference system", 4);
 
@@ -514,12 +528,14 @@ static void parse_gml_srs(xmlNodePtr xnode, gmlSrs *srs)
 		srs->srid = atoi(++p);
 
 		/* Check into spatial_ref_sys that this SRID really exist */
-		is_planar = gml_is_srid_planar(srs->srid);
-		if (srs->srid == SRID_UNKNOWN || is_planar == -1)
+		is_axis_order_gis_friendly = gml_is_srs_axis_order_gis_friendly(srs->srid);
+		if (srs->srid == SRID_UNKNOWN || is_axis_order_gis_friendly == -1)
 			gml_lwpgerror("unknown spatial reference system", 6);
 
-		/* About lat/lon issue, Cf: http://tinyurl.com/yjpr55z */
-		srs->reverse_axis = !is_planar && latlon;
+		/* Reverse axis order if the srsName is meant to honour the axis
+		   order defined by the authority and if that axis order is not
+		   the GIS friendly one. */
+		srs->reverse_axis = !is_axis_order_gis_friendly && honours_authority_axis_order;
 
 		xmlFree(srsname);
 		return;
@@ -611,7 +627,7 @@ static POINTARRAY* parse_gml_coordinates(xmlNodePtr xnode, bool *hasz)
 	int gml_dims;
 	char *p, *q;
 	bool digit;
-	POINT4D pt;
+	POINT4D pt = {0};
 
 	/* We begin to retrieve coordinates string */
 	gml_coord = xmlNodeGetContent(xnode);
@@ -624,7 +640,7 @@ static POINTARRAY* parse_gml_coordinates(xmlNodePtr xnode, bool *hasz)
 	 */
 
 	/* Retrieve separator between coordinates tuples */
-	gml_ts = gmlGetProp(xnode, "ts");
+	gml_ts = gmlGetProp(xnode, (xmlChar *) "ts");
 	if (gml_ts == NULL) ts = ' ';
 	else
 	{
@@ -635,7 +651,7 @@ static POINTARRAY* parse_gml_coordinates(xmlNodePtr xnode, bool *hasz)
 	}
 
 	/* Retrieve separator between each coordinate */
-	gml_cs = gmlGetProp(xnode, "cs");
+	gml_cs = gmlGetProp(xnode, (xmlChar *) "cs");
 	if (gml_cs == NULL) cs = ',';
 	else
 	{
@@ -646,7 +662,7 @@ static POINTARRAY* parse_gml_coordinates(xmlNodePtr xnode, bool *hasz)
 	}
 
 	/* Retrieve decimal separator */
-	gml_dec = gmlGetProp(xnode, "decimal");
+	gml_dec = gmlGetProp(xnode, (xmlChar *) "decimal");
 	if (gml_dec == NULL) dec = '.';
 	else
 	{
@@ -725,7 +741,7 @@ static POINTARRAY* parse_gml_coord(xmlNodePtr xnode, bool *hasz)
 	POINTARRAY *dpa;
 	bool x,y,z;
 	xmlChar *c;
-	POINT4D p;
+	POINT4D p = {0};
 
 	/* HasZ?, !HasM, 1 Point */
 	dpa = ptarray_construct_empty(1, 0, 1);
@@ -736,7 +752,7 @@ static POINTARRAY* parse_gml_coord(xmlNodePtr xnode, bool *hasz)
 		if (xyz->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xyz, false)) continue;
 
-		if (is_gml_element(xyz, "X"))
+		if (!strcmp((char *) xyz->name, "X"))
 		{
 			if (x) gml_lwpgerror("invalid GML representation", 21);
 			c = xmlNodeGetContent(xyz);
@@ -744,7 +760,7 @@ static POINTARRAY* parse_gml_coord(xmlNodePtr xnode, bool *hasz)
 			x = true;
 			xmlFree(c);
 		}
-		else  if (is_gml_element(xyz, "Y"))
+		else  if (!strcmp((char *) xyz->name, "Y"))
 		{
 			if (y) gml_lwpgerror("invalid GML representation", 22);
 			c = xmlNodeGetContent(xyz);
@@ -752,7 +768,7 @@ static POINTARRAY* parse_gml_coord(xmlNodePtr xnode, bool *hasz)
 			y = true;
 			xmlFree(c);
 		}
-		else if (is_gml_element(xyz, "Z"))
+		else if (!strcmp((char *) xyz->name, "Z"))
 		{
 			if (z) gml_lwpgerror("invalid GML representation", 23);
 			c = xmlNodeGetContent(xyz);
@@ -786,9 +802,9 @@ static POINTARRAY* parse_gml_pos(xmlNodePtr xnode, bool *hasz)
 	/* HasZ, !HasM, 1 Point */
 	dpa = ptarray_construct_empty(1, 0, 1);
 
-    dimension = gmlGetProp(xnode, "srsDimension");
+    dimension = gmlGetProp(xnode, (xmlChar *) "srsDimension");
     if (dimension == NULL) /* in GML 3.0.0 it was dimension */
-        dimension = gmlGetProp(xnode, "dimension");
+        dimension = gmlGetProp(xnode, (xmlChar *) "dimension");
     if (dimension == NULL) dim = 2;	/* We assume that we are in 2D */
     else
     {
@@ -851,9 +867,9 @@ static POINTARRAY* parse_gml_poslist(xmlNodePtr xnode, bool *hasz)
 	bool digit;
 
 	/* Retrieve gml:srsDimension attribute if any */
-	dimension = gmlGetProp(xnode, "srsDimension");
+	dimension = gmlGetProp(xnode, (xmlChar *) "srsDimension");
 	if (dimension == NULL) /* in GML 3.0.0 it was dimension */
-		dimension = gmlGetProp(xnode, "dimension");
+		dimension = gmlGetProp(xnode, (xmlChar *) "dimension");
 	if (dimension == NULL) dim = 2;	/* We assume that we are in common 2D */
 	else
 	{
@@ -933,43 +949,44 @@ static POINTARRAY* parse_gml_data(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		if (!is_gml_namespace(xa, false)) continue;
 		if (xa->name == NULL) continue;
 
-		if (is_gml_element(xa, "pos"))
+		if (!strcmp((char *) xa->name, "pos"))
 		{
 			tmp_pa = parse_gml_pos(xa, hasz);
 			if (pa == NULL) pa = tmp_pa;
 			else pa = ptarray_merge(pa, tmp_pa);
 
 		}
-		else if (is_gml_element(xa, "posList"))
+		else if (!strcmp((char *) xa->name, "posList"))
 		{
 			tmp_pa = parse_gml_poslist(xa, hasz);
 			if (pa == NULL) pa = tmp_pa;
 			else pa = ptarray_merge(pa, tmp_pa);
 
 		}
-		else if (is_gml_element(xa, "coordinates"))
+		else if (!strcmp((char *) xa->name, "coordinates"))
 		{
 			tmp_pa = parse_gml_coordinates(xa, hasz);
 			if (pa == NULL) pa = tmp_pa;
 			else pa = ptarray_merge(pa, tmp_pa);
 
 		}
-		else if (is_gml_element(xa, "coord"))
+		else if (!strcmp((char *) xa->name, "coord"))
 		{
 			tmp_pa = parse_gml_coord(xa, hasz);
 			if (pa == NULL) pa = tmp_pa;
 			else pa = ptarray_merge(pa, tmp_pa);
 
 		}
-		else if (is_gml_element(xa, "pointRep") ||
-		         is_gml_element(xa, "pointProperty"))
+		else if (!strcmp((char *) xa->name, "pointRep") ||
+		         !strcmp((char *) xa->name, "pointProperty"))
 		{
+
 			found = false;
 			for (xb = xa->children ; xb != NULL ; xb = xb->next)
 			{
 				if (xb->type != XML_ELEMENT_NODE) continue;
 				if (!is_gml_namespace(xb, false)) continue;
-				if (is_gml_element(xb, "Point"))
+				if (!strcmp((char *) xb->name, "Point"))
 				{
 					found = true;
 					break;
@@ -1094,7 +1111,7 @@ static LWGEOM* parse_gml_curve(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "segments"))
+		if (!strcmp((char *) xa->name, "segments"))
 		{
 			found = true;
 			break;
@@ -1109,11 +1126,10 @@ static LWGEOM* parse_gml_curve(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-
-		if (!is_gml_element(xa, "LineStringSegment")) continue;
+		if (strcmp((char *) xa->name, "LineStringSegment")) continue;
 
 		/* GML SF is restricted to linear interpolation  */
-		interpolation = gmlGetProp(xa, "interpolation");
+		interpolation = gmlGetProp(xa, (xmlChar *) "interpolation");
 		if (interpolation != NULL)
 		{
 			if (strcmp((char *) interpolation, "linear"))
@@ -1240,15 +1256,14 @@ static LWGEOM* parse_gml_polygon(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* Polygon/exterior        -> GML 3.1.1 */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!(is_gml_element(xa, "outerBoundaryIs") ||
-		      is_gml_element(xa, "exterior")))
-			continue;
+		if  (strcmp((char *) xa->name, "outerBoundaryIs") &&
+		        strcmp((char *) xa->name, "exterior")) continue;
 
 		for (xb = xa->children ; xb != NULL ; xb = xb->next)
 		{
 			if (xb->type != XML_ELEMENT_NODE) continue;
 			if (!is_gml_namespace(xb, false)) continue;
-			if (!is_gml_element(xb, "LinearRing")) continue;
+			if (strcmp((char *) xb->name, "LinearRing")) continue;
 
 			ppa = (POINTARRAY**) lwalloc(sizeof(POINTARRAY*));
 			ppa[0] = parse_gml_data(xb->children, hasz, root_srid);
@@ -1272,15 +1287,14 @@ static LWGEOM* parse_gml_polygon(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* Polygon/interior        -> GML 3.1.1 */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!(is_gml_element(xa, "innerBoundaryIs") ||
-		      is_gml_element(xa, "interior")))
-			continue;
+		if  (strcmp((char *) xa->name, "innerBoundaryIs") &&
+		        strcmp((char *) xa->name, "interior")) continue;
 
 		for (xb = xa->children ; xb != NULL ; xb = xb->next)
 		{
 			if (xb->type != XML_ELEMENT_NODE) continue;
 			if (!is_gml_namespace(xb, false)) continue;
-			if (!is_gml_element(xb, "LinearRing")) continue;
+			if (strcmp((char *) xb->name, "LinearRing")) continue;
 
 			ppa = (POINTARRAY**) lwrealloc((POINTARRAY *) ppa,
 			                               sizeof(POINTARRAY*) * (ring + 1));
@@ -1329,7 +1343,7 @@ static LWGEOM* parse_gml_triangle(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	/* GML SF is restricted to planar interpolation
 	       NOTA: I know Triangle is not part of SF, but
 	       we have to be consistent with other surfaces */
-	interpolation = gmlGetProp(xnode, "interpolation");
+	interpolation = gmlGetProp(xnode, (xmlChar *) "interpolation");
 	if (interpolation != NULL)
 	{
 		if (strcmp((char *) interpolation, "planar"))
@@ -1344,14 +1358,14 @@ static LWGEOM* parse_gml_triangle(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* Triangle/exterior */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "exterior")) continue;
+		if (strcmp((char *) xa->name, "exterior")) continue;
 
 		for (xb = xa->children ; xb != NULL ; xb = xb->next)
 		{
 			/* Triangle/exterior/LinearRing */
 			if (xb->type != XML_ELEMENT_NODE) continue;
 			if (!is_gml_namespace(xb, false)) continue;
-			if (!is_gml_element(xb, "LinearRing")) continue;
+			if (strcmp((char *) xb->name, "LinearRing")) continue;
 
 			pa = (POINTARRAY*) lwalloc(sizeof(POINTARRAY));
 			pa = parse_gml_data(xb->children, hasz, root_srid);
@@ -1390,11 +1404,11 @@ static LWGEOM* parse_gml_patch(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	gmlSrs srs;
 
 	/* PolygonPatch */
-	if (!is_gml_element(xnode, "PolygonPatch"))
+	if (strcmp((char *) xnode->name, "PolygonPatch"))
 		gml_lwpgerror("invalid GML representation", 48);
 
 	/* GML SF is restricted to planar interpolation  */
-	interpolation = gmlGetProp(xnode, "interpolation");
+	interpolation = gmlGetProp(xnode, (xmlChar *) "interpolation");
 	if (interpolation != NULL)
 	{
 		if (strcmp((char *) interpolation, "planar"))
@@ -1408,14 +1422,14 @@ static LWGEOM* parse_gml_patch(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	for (xa = xnode->children ; xa != NULL ; xa = xa->next)
 	{
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "exterior")) continue;
+		if (strcmp((char *) xa->name, "exterior")) continue;
 
 		/* PolygonPatch/exterior/LinearRing */
 		for (xb = xa->children ; xb != NULL ; xb = xb->next)
 		{
 			if (xb->type != XML_ELEMENT_NODE) continue;
 			if (!is_gml_namespace(xb, false)) continue;
-			if (!is_gml_element(xb, "LinearRing")) continue;
+			if (strcmp((char *) xb->name, "LinearRing")) continue;
 
 			ppa = (POINTARRAY**) lwalloc(sizeof(POINTARRAY*));
 			ppa[0] = parse_gml_data(xb->children, hasz, root_srid);
@@ -1439,13 +1453,13 @@ static LWGEOM* parse_gml_patch(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "interior")) continue;
+		if (strcmp((char *) xa->name, "interior")) continue;
 
 		/* PolygonPatch/interior/LinearRing */
 		for (xb = xa->children ; xb != NULL ; xb = xb->next)
 		{
 			if (xb->type != XML_ELEMENT_NODE) continue;
-			if (!is_gml_element(xb, "LinearRing")) continue;
+			if (strcmp((char *) xb->name, "LinearRing")) continue;
 
 			ppa = (POINTARRAY**) lwrealloc((POINTARRAY *) ppa,
 			                               sizeof(POINTARRAY*) * (ring + 1));
@@ -1494,7 +1508,7 @@ static LWGEOM* parse_gml_surface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "patches"))
+		if (!strcmp((char *) xa->name, "patches"))
 		{
 			found = true;
 			break;
@@ -1507,7 +1521,7 @@ static LWGEOM* parse_gml_surface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "PolygonPatch")) continue;
+		if (strcmp((char *) xa->name, "PolygonPatch")) continue;
 		patch++;
 
 		/* SQL/MM define ST_CurvePolygon as a single patch only,
@@ -1555,8 +1569,8 @@ static LWGEOM* parse_gml_tin(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "patches") ||
-		    is_gml_element(xa, "trianglePatches"))
+		if (!strcmp((char *) xa->name, "patches") ||
+		        !strcmp((char *) xa->name, "trianglePatches"))
 		{
 			found = true;
 			break;
@@ -1569,7 +1583,7 @@ static LWGEOM* parse_gml_tin(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "Triangle")) continue;
+		if (strcmp((char *) xa->name, "Triangle")) continue;
 
 		if (xa->children != NULL)
 			geom = (LWGEOM*) lwtin_add_lwtriangle((LWTIN *) geom,
@@ -1605,7 +1619,7 @@ static LWGEOM* parse_gml_mpoint(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* MultiPoint/pointMember */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "pointMembers"))
+		if (!strcmp((char *) xa->name, "pointMembers"))
 		{
 			for (xb = xa->children ; xb != NULL ; xb = xb->next)
 			{
@@ -1614,7 +1628,7 @@ static LWGEOM* parse_gml_mpoint(xmlNodePtr xnode, bool *hasz, int *root_srid)
 				    (LWPOINT*)parse_gml(xb, hasz, root_srid));
 			}
 		}
-		else if (is_gml_element(xa, "pointMember"))
+		else if (!strcmp((char *) xa->name, "pointMember"))
 		{
 			if (xa->children != NULL)
 				geom = (LWGEOM*)lwmpoint_add_lwpoint((LWMPOINT*)geom,
@@ -1651,7 +1665,7 @@ static LWGEOM* parse_gml_mline(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* MultiLineString/lineStringMember */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "lineStringMember")) continue;
+		if (strcmp((char *) xa->name, "lineStringMember")) continue;
 		if (xa->children != NULL)
 			geom = (LWGEOM*)lwmline_add_lwline((LWMLINE*)geom,
 			                                   (LWLINE*)parse_gml(xa->children, hasz, root_srid));
@@ -1687,7 +1701,7 @@ static LWGEOM* parse_gml_mcurve(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* MultiCurve/curveMember */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "curveMembers"))
+		if (!strcmp((char *) xa->name, "curveMembers"))
 		{
 			for (xb = xa->children ; xb != NULL ; xb = xb->next)
 			{
@@ -1696,7 +1710,7 @@ static LWGEOM* parse_gml_mcurve(xmlNodePtr xnode, bool *hasz, int *root_srid)
 				                                       (LWLINE*)parse_gml(xb, hasz, root_srid));
 			}
 		}
-		else if (is_gml_element(xa, "curveMember"))
+		else if (!strcmp((char *) xa->name, "curveMember"))
 		{
 			if (xa->children != NULL)
 				geom = (LWGEOM*)lwmline_add_lwline((LWMLINE*)geom,
@@ -1733,7 +1747,7 @@ static LWGEOM* parse_gml_mpoly(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* MultiPolygon/polygonMember */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "polygonMember")) continue;
+		if (strcmp((char *) xa->name, "polygonMember")) continue;
 		if (xa->children != NULL)
 			geom = (LWGEOM*)lwmpoly_add_lwpoly((LWMPOLY*)geom,
 			                                   (LWPOLY*)parse_gml(xa->children, hasz, root_srid));
@@ -1768,7 +1782,7 @@ static LWGEOM* parse_gml_msurface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* MultiSurface/surfaceMember */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "surfaceMembers"))
+		if (!strcmp((char *) xa->name, "surfaceMembers"))
 		{
 			for (xb = xa->children ; xb != NULL ; xb = xb->next)
 			{
@@ -1777,7 +1791,7 @@ static LWGEOM* parse_gml_msurface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 				                                       (LWPOLY*)parse_gml(xb, hasz, root_srid));
 			}
 		}
-		else if (is_gml_element(xa, "surfaceMember"))
+		else if (!strcmp((char *) xa->name, "surfaceMember"))
 		{
 			if (xa->children != NULL)
 				geom = (LWGEOM*)lwmpoly_add_lwpoly((LWMPOLY*)geom,
@@ -1816,7 +1830,7 @@ static LWGEOM* parse_gml_psurface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	{
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (is_gml_element(xa, "polygonPatches"))
+		if (!strcmp((char *) xa->name, "polygonPatches"))
 		{
 			found = true;
 			break;
@@ -1829,7 +1843,7 @@ static LWGEOM* parse_gml_psurface(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		/* PolyhedralSurface/polygonPatches/PolygonPatch */
 		if (xa->type != XML_ELEMENT_NODE) continue;
 		if (!is_gml_namespace(xa, false)) continue;
-		if (!is_gml_element(xa, "PolygonPatch")) continue;
+		if (strcmp((char *) xa->name, "PolygonPatch")) continue;
 
 		geom = (LWGEOM*)lwpsurface_add_lwpoly((LWPSURFACE*)geom,
 		                                      (LWPOLY*)parse_gml_patch(xa, hasz, root_srid));
@@ -1869,10 +1883,10 @@ static LWGEOM* parse_gml_coll(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		 * polygonMember are parts of geometryMember
 		 * substitution group
 		 */
-		if (	   is_gml_element(xa, "pointMember")
-		        || is_gml_element(xa, "lineStringMember")
-		        || is_gml_element(xa, "polygonMember")
-		        || is_gml_element(xa, "geometryMember"))
+		if (	   !strcmp((char *) xa->name, "pointMember")
+		        || !strcmp((char *) xa->name, "lineStringMember")
+		        || !strcmp((char *) xa->name, "polygonMember")
+		        || !strcmp((char *) xa->name, "geometryMember"))
 		{
 			if (xa->children == NULL) break;
 			geom = (LWGEOM*)lwcollection_add_lwgeom((LWCOLLECTION *)geom,
@@ -1898,7 +1912,7 @@ lwgeom_from_gml(const char *xml, int xml_size)
 	/* Begin to Parse XML doc */
 	xmlInitParser();
 
-	xmldoc = xmlReadMemory(xml, xml_size, NULL, NULL, 0);
+	xmldoc = xmlReadMemory(xml, xml_size, NULL, NULL, XML_PARSE_SAX1);
 	if (!xmldoc)
 	{
 		xmlCleanupParser();
@@ -1950,12 +1964,8 @@ static LWGEOM* parse_gml(xmlNodePtr xnode, bool *hasz, int *root_srid)
 	xmlNodePtr xa = xnode;
 	gmlSrs srs;
 
-	/* Scroll forward to the root node */
-	while (xa != NULL &&
-	      (xa->type != XML_ELEMENT_NODE || !is_gml_namespace(xa, false)))
-	{
-		xa = xa->next;
-	}
+	while (xa != NULL && (xa->type != XML_ELEMENT_NODE
+	                      || !is_gml_namespace(xa, false))) xa = xa->next;
 
 	if (xa == NULL) gml_lwpgerror("invalid GML representation", 55);
 
@@ -1965,50 +1975,50 @@ static LWGEOM* parse_gml(xmlNodePtr xnode, bool *hasz, int *root_srid)
 		*root_srid = srs.srid;
 	}
 
-	if (is_gml_element(xa, "Point"))
+	if (!strcmp((char *) xa->name, "Point"))
 		return parse_gml_point(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "LineString"))
+	if (!strcmp((char *) xa->name, "LineString"))
 		return parse_gml_line(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "Curve"))
+	if (!strcmp((char *) xa->name, "Curve"))
 		return parse_gml_curve(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "LinearRing"))
+	if (!strcmp((char *) xa->name, "LinearRing"))
 		return parse_gml_linearring(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "Polygon"))
+	if (!strcmp((char *) xa->name, "Polygon"))
 		return parse_gml_polygon(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "Triangle"))
+	if (!strcmp((char *) xa->name, "Triangle"))
 		return parse_gml_triangle(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "Surface"))
+	if (!strcmp((char *) xa->name, "Surface"))
 		return parse_gml_surface(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiPoint"))
+	if (!strcmp((char *) xa->name, "MultiPoint"))
 		return parse_gml_mpoint(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiLineString"))
+	if (!strcmp((char *) xa->name, "MultiLineString"))
 		return parse_gml_mline(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiCurve"))
+	if (!strcmp((char *) xa->name, "MultiCurve"))
 		return parse_gml_mcurve(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiPolygon"))
+	if (!strcmp((char *) xa->name, "MultiPolygon"))
 		return parse_gml_mpoly(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiSurface"))
+	if (!strcmp((char *) xa->name, "MultiSurface"))
 		return parse_gml_msurface(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "PolyhedralSurface"))
+	if (!strcmp((char *) xa->name, "PolyhedralSurface"))
 		return parse_gml_psurface(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "Tin") ||
-	    is_gml_element(xa, "TriangulatedSurface"))
+	if ((!strcmp((char *) xa->name, "Tin")) ||
+	        !strcmp((char *) xa->name, "TriangulatedSurface" ))
 		return parse_gml_tin(xa, hasz, root_srid);
 
-	if (is_gml_element(xa, "MultiGeometry"))
+	if (!strcmp((char *) xa->name, "MultiGeometry"))
 		return parse_gml_coll(xa, hasz, root_srid);
 
 	gml_lwpgerror("invalid GML representation", 56);
